//
// Copyright (c) 2001-2016 by PDFTron Systems Inc. All Rights Reserved.
//

using System;
using System.Collections.Generic;

using pdftron;
using pdftron.Common;
using pdftron.Filters;
using pdftron.SDF;
using pdftron.PDF;

using XSet = System.Collections.Generic.List<int>;

namespace ElementEditTestCS
{
    /// <summary>
    /// The sample code shows how to edit the page display list and how to modify graphics state 
    /// attributes on existing Elements. In particular the sample program strips all images from 
    /// the page, changes path fill color to red, and changes text fill color to blue. 
    /// </summary>
    class ElementEditTest
    {
        private static pdftron.PDFNetLoader pdfNetLoader = pdftron.PDFNetLoader.Instance();

        static string input_path = "../../../../TestFiles/";
        static string output_path = input_path + "Output/";

        static byte[] FlipImage(Element element)
        {
            Image2RGB image2rgb = new Image2RGB(element);
            int width = element.GetImageWidth();
            int height = element.GetImageHeight();
            int out_data_sz = width * height * 3;
            int stride = width * 3;
            FilterReader reader = new FilterReader(image2rgb);
            byte[] image_data = new byte[out_data_sz];
            byte[] flipped_data = new byte[out_data_sz];
            reader.Read(image_data);
            for (int row = 0; row < height; ++row)
            {
                Buffer.BlockCopy(image_data, row * stride, flipped_data, out_data_sz - (stride * (row + 1)), stride);
            }
            return flipped_data;
        }

        static void FilterImagesByRect(Page src_page, List<pdftron.PDF.Rect> clipping_rects)
        {
            ElementReader reader = new ElementReader();
            Element element;
            PDFDoc images_doc = new PDFDoc();

            Page page = images_doc.PageCreate();

            // Create a page
            images_doc.PagePushBack(page);

            //////////////////////////////////////////////////////////////////////////////////////////////////////
            // DEBUG show clipping rect
            foreach (Rect rect in clipping_rects)
            {
                // Dump some rect annots
                pdftron.PDF.Annots.Square square = pdftron.PDF.Annots.Square.Create(images_doc.GetSDFDoc(), rect);
                page.AnnotPushBack(square);
            }
            //////////////////////////////////////////////////////////////////////////////////////////////////////

            reader.Begin(src_page);
            while ((element = reader.Next()) != null) // Read page contents
            {
                switch (element.GetType())
                {
                    case Element.Type.e_image:
                    case Element.Type.e_inline_image:
                        {
                            ElementWriter new_writer = new ElementWriter();
                            ElementBuilder new_builder = new ElementBuilder();
                            new_writer.Begin(page);
                            Matrix2D ctm = element.GetCTM();
                            double x2 = 1, y2 = 1, y1 = ctm.m_v;
                            ctm.Mult(ref x2, ref y2);

                            Console.WriteLine("    Coords: x1={0}, y1={1}, x2={2}, y2={3}", ctm.m_h, ctm.m_v, x2, y2);
                            pdftron.PDF.Rect image_rect = new pdftron.PDF.Rect(ctm.m_h, ctm.m_v, x2, y2);
                            bool conti = false;
                            foreach (Rect rect in clipping_rects)
                            {
                                if (rect.Contains(image_rect.x1, image_rect.y1) && rect.Contains(image_rect.x2, image_rect.y2))
                                {
                                    conti = true;
                                    break;
                                }
                            }

                            if (!conti)
                            {
                                break;
                            }

                            pdftron.PDF.Image image = null;
                            pdftron.SDF.Obj obj = images_doc.GetSDFDoc().ImportObj(element.GetXObject(), true);


                            if (element.GetType() == Element.Type.e_image)
                            {
                                image = new pdftron.PDF.Image(obj);
                            }
                            else // inline image
                            {
                                if (y1 > y2)
                                {
                                    byte[] flipped_image = FlipImage(element);
                                    image = pdftron.PDF.Image.Create(images_doc, flipped_image, element.GetImageWidth(), element.GetImageHeight(), 8, ColorSpace.CreateDeviceRGB());
                                }
                                else
                                {
                                    Image2RGB image2rgb = new Image2RGB(element);
                                    FilterReader image_reader = new FilterReader(image2rgb);
                                    image = pdftron.PDF.Image.Create(images_doc, image_reader, element.GetImageWidth(), element.GetImageHeight(), 8, ColorSpace.CreateDeviceRGB());
                                }
                            }

                            Element new_element = new_builder.CreateImage(image, ctm);
                            new_writer.WritePlacedElement(new_element);
                            new_writer.End();
                        }
                        break;
                    default:
                        break;
                }
            }
            reader.End();
            images_doc.Save(output_path + "simple_out.pdf", SDFDoc.SaveOptions.e_remove_unused | SDFDoc.SaveOptions.e_linearized);
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            PDFNet.Initialize();

            try
            {
                using(PDFDoc doc = new PDFDoc(input_path + "simple.pdf"))
                {
                    List<Rect> list_rects = new List<Rect>();
                    list_rects.Add(new Rect(43.480928, 570.224233, 172.474349, 695.320886));
                    FilterImagesByRect(doc.GetPageIterator().Current(), list_rects);
                }
            }
            catch (PDFNetException e)
            {
                Console.WriteLine(e.GetMessage());
            }
        }
    }
}
